<?php
/*
Plugin Name: Ship Discounts
Description: Offer your customers shipping services with real-time quotes.
Version: 1.0.6.2
License: GPL v2 or later
License URI: https://www.gnu.org/licenses/gpl-2.0.html
Text Domain: ship-discounts-lar
Domain Path: /languages/
WC tested up to: 8.8.3

Ship Discounts is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
any later version.

Ship Discounts is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ship Discounts. If not, see https://www.gnu.org/licenses/gpl-2.0.html.
*/

if (!defined('ABSPATH'))
    exit;

if (!defined('LAR_VERSION'))
    define('LAR_VERSION', '1.0.6.2');

if (!defined('LAR_AUTOMATIC_ORDER'))
    define('LAR_AUTOMATIC_ORDER', false);

if (!defined('LAR_ADVANCED_SETTINGS'))
    define('LAR_ADVANCED_SETTINGS', false);

if (!class_exists('Ship_Discounts_LAR_Plugin')) {
    /**
     * Class of the Ship Discounts plugin.
     */
    class Ship_Discounts_LAR_Plugin {
        /**
         * @var bool|Ship_Discounts_LAR_Plugin Current instance of the class.
         */
        private static $instance = false;

        /**
         * API key parameter.
         */
        const API_PARAM = '?apiKey=';

        /**
         * URL to get the carriers' quotes.
         */
        const CARRIER_QUOTE_URL = 'carrier/quote';

        /**
         * URL to get an order's details.
         */
        const ORDER_URL = 'order';

        /**
         * URL to get an order's label.
         */
        const ORDER_LABEL_URL = 'order/documents';

        /**
         * Constructor.
         */
        private function __construct() {
            if (!function_exists('is_plugin_active'))
                include_once ABSPATH . 'wp-admin/includes/plugin.php';

            if (is_plugin_active('woocommerce/woocommerce.php')) {
                // Languages
                load_plugin_textdomain('ship-discounts-lar', false, dirname(plugin_basename(__FILE__)) . '/languages/');
                add_filter('load_textdomain_mofile', array($this, 'lar_load_textdomain'), 10, 2);

                // Plugin activation
                add_action('admin_menu', array($this, 'register_lar_menu_page'), 999);

                // Token verification (1.0.5)
                if ((get_option("lar_api_dev", "") && !get_option("lar_api_token_dev", "")) ||
                    (!get_option("lar_api_dev", "") && !get_option("lar_api_token_prod", "")))
                    update_option("lar_account_activated", false);

                // Shipping method
                if (get_option("lar_account_activated", false)) {
                    include_once(__DIR__ . "/includes/class-lar-carriers.php");
                    include_once(__DIR__ . "/includes/class-lar-boxes.php");
                    include_once(__DIR__ . "/includes/class-lar-packages.php");
                    include_once(__DIR__ . "/includes/wc-shipping-ship-discounts.php");
                    include_once(__DIR__ . "/includes/wc-settings-section.php");
                    include_once(__DIR__ . "/includes/wc-product-settings.php");
                    include_once(__DIR__ . "/includes/wc-cart-checkout.php");
                    include_once(__DIR__ . "/includes/order-meta-box.php");

                    // Email
                    add_filter('woocommerce_email_format_string', array($this, 'lar_email_placeholders'), 20, 2);
                    add_filter('woocommerce_email_settings', array($this, 'lar_add_placeholders_info'));

                    // Client styles and scripts
                    add_action('init', array($this, 'lar_load_client_styles_scripts'));

                    // API
                    include_once "api/api.php";
                }

                // Admin styles and scripts
                add_action('admin_init', array($this, 'lar_load_admin_styles_scripts'));
            }
        }

        /**
         * Returns the current instance. If it does not exist, it is first created.
         * @return Ship_Discounts_LAR_Plugin Current instance.
         */
        public static function getInstance(): Ship_Discounts_LAR_Plugin {
            if (!self::$instance)
                self::$instance = new self;
            return self::$instance;
        }

        /**
         * Redirects to the installation page after activation.
         * @return void
         */
        public static function lar_activation($plugin) {
            if (!function_exists('is_plugin_active'))
                include_once ABSPATH . 'wp-admin/includes/plugin.php';

            if (is_plugin_active('woocommerce/woocommerce.php')) {
                if ($plugin == plugin_basename(__FILE__)) {
                    // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                    exit(wp_safe_redirect(esc_url(admin_url('admin.php?page=lar-api'))));
                }
            }
        }

        /**
         * Loads the admin styles and scripts.
         * @return void
         */
        function lar_load_admin_styles_scripts() {
            wp_enqueue_style('lar_admin_style', plugins_url('css/admin.css', __FILE__), array(), LAR_VERSION);

            // phpcs:ignore WordPress.Security.NonceVerification.Recommended
            if (is_admin() && $_GET['page'] === "wc-settings" && $_GET['tab'] === "shipping") {
                wp_enqueue_script('lar-admin-script', plugins_url('js/admin.min.js', __FILE__), array('jquery', 'wp-util'), LAR_VERSION, ['in-footer' => false]);
                wp_enqueue_script('lar-jquery-ui-script', plugins_url('js/jquery-ui-1.13.2.custom/jquery-ui.min.js', __FILE__), array('jquery'), LAR_VERSION, ['in-footer' => false]);

                wp_localize_script('lar-admin-script', 'ajax_var', array(
                    'url'   => admin_url('admin-ajax.php'),
                    'nonce' => wp_create_nonce('lar-ajax-nonce')
                ));
            }
        }

        /**
         * Loads the client styles and scripts.
         * @return void
         */
        function lar_load_client_styles_scripts() {
            wp_enqueue_style('lar_client_style', plugins_url('css/style.css', __FILE__), array(), LAR_VERSION);
        }

        /**
         * Loads the right language from languages directory.
         * @param string $mofile Language file path.
         * @param string $domain Text domain.
         * @return string Language file path.
         */
        function lar_load_textdomain($mofile, $domain) {
            if ('ship-discounts-lar' === $domain)
                return ABSPATH . DIRECTORY_SEPARATOR . 'wp-content' . DIRECTORY_SEPARATOR . 'plugins/' . dirname(plugin_basename(__FILE__)) . '/languages/lar-' . get_locale() . '.mo';

            return $mofile;
        }

        /**
         * Registers the Ship Discounts API submenu.
         * @return void
         */
        function register_lar_menu_page() {
            add_submenu_page('woocommerce', esc_html__('Ship Discounts API', 'ship-discounts-lar'), esc_html__('Ship Discounts API', 'ship-discounts-lar'), 'manage_woocommerce', 'lar-api', array($this, 'lar_admin_api_page'), 999);
        }

        /**
         * Displays the Ship Discounts API page.
         * @return void
         */
        function lar_admin_api_page() {
            include("page-settings.php");
        }

        /**
         * Adds custom placeholders to WooCommerce email subject and body.
         * @param string $string
         * @param object $email
         * @return string
         */
        function lar_email_placeholders($string, $email) {
            $order = $email->object;

            $tracking = '';
            if ($order->get_meta('_lar_tracking_nb'))
                $tracking = $order->get_meta('_lar_tracking_nb');
            if ($order->get_meta('_lar_tracking_url')) {
                if (!$tracking) $tracking = esc_html__('Track my order', 'ship-discounts-lar');
                $tracking = '<a href="' . $order->get_meta('_lar_tracking_url') . '" target="_blank">' . $tracking . '</a>';
            }

            $carrier_code = $order->get_meta('_lar_carrier_code') ?: '';
            $service_code = $order->get_meta('_lar_service_code') ?: '';

            $new_placeholders = array(
                '{tracking}'     => $tracking,
                '{carrier_code}' => $carrier_code,
                '{service_code}' => $service_code,
            );

            return str_replace(array_keys($new_placeholders), array_values($new_placeholders), $string);
        }

        /**
         * Specifies the available email placeholders.
         * @param $settings array Email settings.
         * @return array Email settings with the available email placeholders.
         */
        function lar_add_placeholders_info($settings) {
            $key = array_search('email_notification_settings-description', array_column($settings, 'id')) + 1;

            $desc = esc_html__("Since the plugin Ship Discounts is activated, you have access to the following placeholders: ", 'ship-discounts-lar');
            $desc .= '<br><b>{tracking}</b> => ' . esc_html__("Displays the tracking code with the URL", 'ship-discounts-lar');
            $desc .= '<br><b>{carrier_code}</b> => ' . esc_html__("Displays the carrier code", 'ship-discounts-lar');
            $desc .= '<br><b>{service_code}</b> => ' . esc_html__("Displays the service code", 'ship-discounts-lar');

            $custom_setting[] = array(
                'title' => esc_html__("Ship Discounts placeholders", 'ship-discounts-lar'),
                'desc'  => $desc,
                'id'    => 'lar_placeholders',
                'type'  => 'title',
            );
            return array_merge(array_slice($settings, 0, $key), $custom_setting, array_slice($settings, $key, count($settings) - 1));
        }

        /**
         * Converts to pounds.
         * @param $unit string Unit to convert from.
         * @param $value float|int|string Value to convert.
         * @return float|int|string Value in pounds.
         */
        public static function convertToLbs($unit, $value) {
            if (!is_numeric($value)) return 0;

            switch ($unit) {
                case 'kg':
                    return $value * 2.20462262185;
                case 'g':
                    return $value * 0.0022046226;
                case 'oz':
                    return $value * 0.0625;
                default:
                    return $value;
            }
        }

        /**
         * Converts to inches.
         * @param $unit string Unit to convert from.
         * @param $value float|int|string Value to convert.
         * @return float|int|string Value in inches.
         */
        public static function convertToIn($unit, $value) {
            if (!is_numeric($value)) return 0;

            switch ($unit) {
                case 'm':
                    return $value * 39.37007874;
                case 'cm':
                    return $value * 0.3937007874;
                case 'mm':
                    return $value * 0.03937007874;
                case 'yd':
                    return $value * 36;
                default:
                    return $value;
            }
        }

        /**
         * Checks if an array key exists in a multidimensional array.
         * @param $array array Array.
         * @param $key mixed Key.
         * @return bool If the key exists.
         */
        public static function multiArrayKeyExists($array, $key) {
            if (array_key_exists($key, $array))
                return true;

            foreach ($array as $a) {
                if (is_array($a)) {
                    if (self::multiArrayKeyExists($a, $key))
                        return true;
                }
            }

            return false;
        }

        /**
         * Checks if a product is missing its weight, dimensions and/or SKU.
         * @param $product mixed Product.
         * @return array|null Array of missing data or null.
         */
        public static function lar_check_product_missing($product) {
            $weight = $product->get_weight() ? Ship_Discounts_LAR_Plugin::convertToLbs(get_option('woocommerce_weight_unit'), $product->get_weight()) : false;
            if ($weight) $weight = $weight >= 0.1 && $weight <= 150;

            $height = $product->get_height() ? Ship_Discounts_LAR_Plugin::convertToIn(get_option('woocommerce_dimension_unit'), $product->get_height()) : false;
            if ($height) $height = $height >= 0.1 && $height <= 144;

            $width = $product->get_width() ? Ship_Discounts_LAR_Plugin::convertToIn(get_option('woocommerce_dimension_unit'), $product->get_width()) : false;
            if ($width) $width = $width >= 0.1 && $width <= 144;

            $length = $product->get_length() ? Ship_Discounts_LAR_Plugin::convertToIn(get_option('woocommerce_dimension_unit'), $product->get_length()) : false;
            if ($length) $length = $length >= 0.1 && $length <= 144;

            $sku = $product->get_sku() ?: false;

            if (!$weight || !$height || !$width || !$length || !$sku) {
                $missing = "";

                if (!$weight) $missing .= esc_html__("weight", 'ship-discounts-lar') . ', ';
                if (!$height) $missing .= esc_html__("height", 'ship-discounts-lar') . ', ';
                if (!$width) $missing .= esc_html__("width", 'ship-discounts-lar') . ', ';
                if (!$length) $missing .= esc_html__("length", 'ship-discounts-lar') . ', ';
                if (!$sku) $missing .= esc_html__("SKU", 'ship-discounts-lar') . ', ';
                $missing = substr($missing, 0, strlen($missing) - 2);

                return array(
                    "id"      => $product->get_id(),
                    "parent"  => $product->get_parent_id(),
                    "name"    => $product->get_name(),
                    "missing" => $missing,
                );
            }

            return null;
        }

        /**
         * Create an array of WC_Order_Item_Product based on items in cart.
         * @param array $cart_items Items in cart.
         * @return array Array of WC_Order_Item_Product.
         */
        public static function lar_cart_items_to_item_products($cart_items) {
            $products = [];

            if ($cart_items) {
                foreach ($cart_items as $cart_item) {
                    $item = new WC_Order_Item_Product();
                    $item->set_props(array(
                        'quantity'     => $cart_item['quantity'],
                        'variation'    => $cart_item['variation'],
                        'subtotal'     => $cart_item['line_subtotal'],
                        'total'        => $cart_item['line_total'],
                        'subtotal_tax' => $cart_item['line_subtotal_tax'],
                        'total_tax'    => $cart_item['line_tax'],
                        'taxes'        => $cart_item['line_tax_data'],
                    ));

                    $product = $cart_item['data'];
                    if ($product) {
                        $item->set_props(array(
                            'name'         => $product->get_name(),
                            'tax_class'    => $product->get_tax_class(),
                            'product_id'   => $product->is_type('variation') ? $product->get_parent_id() : $product->get_id(),
                            'variation_id' => $product->is_type('variation') ? $product->get_id() : 0,
                        ));
                    }
                    $item->set_backorder_meta();

                    $products[] = $item;
                }
            }

            return $products;
        }
    }

    add_action('activated_plugin', array('Ship_Discounts_LAR_Plugin', 'lar_activation'));

    add_action('before_woocommerce_init', function () {
        if (class_exists('\Automattic\WooCommerce\Utilities\FeaturesUtil'))
            \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility('cart_checkout_blocks', __FILE__, false);

        if (class_exists(\Automattic\WooCommerce\Utilities\FeaturesUtil::class))
            \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility('custom_order_tables', __FILE__, true);
    });

    $Ship_Discounts_LAR_Plugin = Ship_Discounts_LAR_Plugin::getInstance();
}